export default commands;
export type ContentTypeEnum = "plaintext" | "image";
export type SetClipboardOptions = {
    /**
     * base64-encoded clipboard content to set
     */
    b64Content: string;
    /**
     * [text] The clipboard content type to set
     */
    contentType: ContentTypeEnum;
};
export type GetClipboardOptions = {
    /**
     * [plaintext] The clipboard content type to get.
     * Only PNG images are supported for extraction if set to 'image'.
     */
    contentType: ContentTypeEnum;
};
export namespace commands {
    /**
     * @typedef {Object} SetClipboardOptions
     * @property {string} b64Content base64-encoded clipboard content to set
     * @property {ContentTypeEnum} contentType [text] The clipboard content type to set
     */
    /**
     * Sets the Windows clipboard to the given string or PNG-image.
     *
     * @param {Partial<SetClipboardOptions>} opts
     */
    function windowsSetClipboard(opts?: Partial<SetClipboardOptions>): Promise<import("teen_process").TeenProcessExecResult<string>>;
    /**
     * @typedef {Object} GetClipboardOptions
     * @property {ContentTypeEnum} contentType [plaintext] The clipboard content type to get.
     * Only PNG images are supported for extraction if set to 'image'.
     */
    /**
     * Returns the Windows clipboard content as base64-encoded string.
     *
     * @param {Partial<GetClipboardOptions>} opts
     * @returns {Promise<string>} base64-encoded content of the clipboard
     */
    function windowsGetClipboard(opts?: Partial<GetClipboardOptions>): Promise<string>;
}
//# sourceMappingURL=clipboard.d.ts.map