"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = __importDefault(require("lodash"));
const driver_1 = require("appium/driver");
const constants_1 = require("../constants");
const commands = {};
const POWER_SHELL_SCRIPT_PATTERN = /^powerShell$/;
const WINDOWS_EXTENSION_SCRIPT_PATTERN = /^windows:/;
const EXTENSION_COMMANDS_MAPPING = {
    startRecordingScreen: 'startRecordingScreen',
    stopRecordingScreen: 'stopRecordingScreen',
    launchApp: 'windowsLaunchApp',
    closeApp: 'windowsCloseApp',
    deleteFile: 'windowsDeleteFile',
    deleteFolder: 'windowsDeleteFolder',
    click: 'windowsClick',
    scroll: 'windowsScroll',
    clickAndDrag: 'windowsClickAndDrag',
    hover: 'windowsHover',
    keys: 'windowsKeys',
    setClipboard: 'windowsSetClipboard',
    getClipboard: 'windowsGetClipboard',
};
commands.execute = async function execute(script, args) {
    if (WINDOWS_EXTENSION_SCRIPT_PATTERN.test(script)) {
        this.log.info(`Executing extension command '${script}'`);
        script = script.replace(WINDOWS_EXTENSION_SCRIPT_PATTERN, '').trim();
        return await this.executeWindowsCommand(script, lodash_1.default.isArray(args) ? args[0] : args);
    }
    else if (POWER_SHELL_SCRIPT_PATTERN.test(script)) {
        this.ensureFeatureEnabled(constants_1.POWER_SHELL_FEATURE);
        return await this.execPowerShell(lodash_1.default.isArray(args) ? lodash_1.default.first(args) : args);
    }
    throw new driver_1.errors.NotImplementedError();
};
commands.executeWindowsCommand = async function executeWindowsCommand(command, opts = {}) {
    if (!lodash_1.default.has(EXTENSION_COMMANDS_MAPPING, command)) {
        throw new driver_1.errors.UnknownCommandError(`Unknown windows command '${command}'. ` +
            `Only ${lodash_1.default.keys(EXTENSION_COMMANDS_MAPPING)} commands are supported.`);
    }
    return await this[EXTENSION_COMMANDS_MAPPING[command]](opts);
};
exports.default = commands;
//# sourceMappingURL=execute.js.map