export default commands;
export type DeleteFileOptions = {
    /**
     * - The path to a file.
     * The path may contain environment variables that could be expanded on the server side.
     * Due to security reasons only variables listed below would be expanded: `APPDATA`,
     * `LOCALAPPDATA`, `PROGRAMFILES`, `PROGRAMFILES(X86)`, `PROGRAMDATA`, `ALLUSERSPROFILE`,
     * `TEMP`, `TMP`, `HOMEPATH`, `USERPROFILE`, `PUBLIC`.
     */
    remotePath: string;
};
export type DeleteFolderOptions = {
    /**
     * - The path to a folder.
     * The path may contain environment variables that could be expanded on the server side.
     * Due to security reasons only variables listed below would be expanded: `APPDATA`,
     * `LOCALAPPDATA`, `PROGRAMFILES`, `PROGRAMFILES(X86)`, `PROGRAMDATA`, `ALLUSERSPROFILE`,
     * `TEMP`, `TMP`, `HOMEPATH`, `USERPROFILE`, `PUBLIC`.
     */
    remotePath: string;
};
export namespace commands {
    function pushFile(remotePath: any, base64Data: any): Promise<void>;
    function pullFile(remotePath: any): Promise<string>;
    function pullFolder(remotePath: any): Promise<string>;
    /**
     * @typedef {Object} DeleteFileOptions
     * @property {string} remotePath - The path to a file.
     * The path may contain environment variables that could be expanded on the server side.
     * Due to security reasons only variables listed below would be expanded: `APPDATA`,
     * `LOCALAPPDATA`, `PROGRAMFILES`, `PROGRAMFILES(X86)`, `PROGRAMDATA`, `ALLUSERSPROFILE`,
     * `TEMP`, `TMP`, `HOMEPATH`, `USERPROFILE`, `PUBLIC`.
     */
    /**
     * Remove the file from the file system
     *
     * @param {DeleteFileOptions} opts
     * @throws {InvalidArgumentError} If the file to be deleted does not exist or
     * remote path is not an absolute path.
     */
    function windowsDeleteFile(opts: DeleteFileOptions): Promise<void>;
    /**
     * @typedef {Object} DeleteFolderOptions
     * @property {string} remotePath - The path to a folder.
     * The path may contain environment variables that could be expanded on the server side.
     * Due to security reasons only variables listed below would be expanded: `APPDATA`,
     * `LOCALAPPDATA`, `PROGRAMFILES`, `PROGRAMFILES(X86)`, `PROGRAMDATA`, `ALLUSERSPROFILE`,
     * `TEMP`, `TMP`, `HOMEPATH`, `USERPROFILE`, `PUBLIC`.
     */
    /**
     * Remove the folder from the file system
     *
     * @param {DeleteFolderOptions} opts
     * @throws {InvalidArgumentError} If the folder to be deleted does not exist or
     * remote path is not an absolute path.
     */
    function windowsDeleteFolder(opts: DeleteFolderOptions): Promise<void>;
}
//# sourceMappingURL=file-movement.d.ts.map