export default commands;
export type ExecPowerShellOptions = {
    /**
     * A valid Power Shell script to execute
     */
    script: string | null;
    /**
     * A valid Power Shell command to execute
     */
    command: string | null;
};
export namespace commands {
    /**
     * @typedef {Object} ExecPowerShellOptions
     * @property {string?} script A valid Power Shell script to execute
     * @property {string?} command A valid Power Shell command to execute
     */
    /**
     * Executes the given Power Shell command or a whole script based on the
     * given options. Either of these options must be provided. If both are provided
     * then the `command` one gets the priority.
     * Note that Power Shell command cannot contain line breaks. Consider making it
     * to a script in such case.
     * Note that by default Power Shell blocks scripts execution, so the script must
     * temporarily switch user execution policy if necessary and restore it afterwards.
     * This makes scripts slightly less performant, as single commands.
     *
     * @param {ExecPowerShellOptions} opts
     * @returns {Promise<string>} The actual stdout of the given command/script
     * @throws {Error} If the exit code of the given command/script is not zero.
     * The actual stderr output is set to the error message value.
     */
    function execPowerShell(opts: ExecPowerShellOptions): Promise<string>;
}
//# sourceMappingURL=powershell.d.ts.map