export default WindowsDriver;
export type WindowsDriverConstraints = typeof desiredCapConstraints;
export type WindowsDriverOpts = import("@appium/types").DriverOpts<WindowsDriverConstraints>;
export class WindowsDriver extends BaseDriver<any, import("@appium/types").StringRecord, import("@appium/types").StringRecord, import("@appium/types").DefaultCreateSessionResult<any>, void, import("@appium/types").StringRecord> {
    static newMethodMap: {
        readonly '/session/:sessionId/appium/start_recording_screen': {
            readonly POST: {
                readonly command: "startRecordingScreen";
                readonly payloadParams: {
                    readonly optional: readonly ["options"];
                };
            };
        };
        readonly '/session/:sessionId/appium/stop_recording_screen': {
            readonly POST: {
                readonly command: "stopRecordingScreen";
                readonly payloadParams: {
                    readonly optional: readonly ["options"];
                };
            };
        };
        readonly '/session/:sessionId/appium/device/push_file': {
            readonly POST: {
                readonly command: "pushFile";
                readonly payloadParams: {
                    readonly required: readonly ["path", "data"];
                };
            };
        };
        readonly '/session/:sessionId/appium/device/pull_file': {
            readonly POST: {
                readonly command: "pullFile";
                readonly payloadParams: {
                    readonly required: readonly ["path"];
                };
            };
        };
        readonly '/session/:sessionId/appium/device/pull_folder': {
            readonly POST: {
                readonly command: "pullFolder";
                readonly payloadParams: {
                    readonly required: readonly ["path"];
                };
            };
        };
        readonly '/session/:sessionId/window/:windowhandle/size': {
            readonly GET: {
                readonly command: "getWindowSize";
            };
            readonly POST: {};
        };
        readonly '/session/:sessionId/window/:windowhandle/position': {
            readonly POST: {};
            readonly GET: {};
        };
        readonly '/session/:sessionId/window_handle': {
            readonly GET: {
                readonly command: "getWindowHandle";
            };
        };
        readonly '/session/:sessionId/window_handles': {
            readonly GET: {
                readonly command: "getWindowHandles";
            };
        };
        readonly '/session/:sessionId/appium/app/launch': {
            readonly POST: {
                readonly command: "launchApp";
            };
        };
        readonly '/session/:sessionId/appium/app/close': {
            readonly POST: {
                readonly command: "closeApp";
            };
        };
        readonly '/session/:sessionId/click': {
            readonly POST: {
                readonly command: "clickCurrent";
                readonly payloadParams: {
                    readonly optional: readonly ["button"];
                };
            };
        };
        readonly '/session/:sessionId/buttondown': {
            readonly POST: {
                readonly command: "buttonDown";
                readonly payloadParams: {
                    readonly optional: readonly ["button"];
                };
            };
        };
        readonly '/session/:sessionId/buttonup': {
            readonly POST: {
                readonly command: "buttonUp";
                readonly payloadParams: {
                    readonly optional: readonly ["button"];
                };
            };
        };
        readonly '/session/:sessionId/doubleclick': {
            readonly POST: {
                readonly command: "doubleClick";
            };
        };
        readonly '/session/:sessionId/touch/click': {
            readonly POST: {
                readonly command: "click";
                readonly payloadParams: {
                    readonly required: readonly ["element"];
                };
            };
        };
        readonly '/session/:sessionId/touch/down': {
            readonly POST: {
                readonly command: "touchDown";
                readonly payloadParams: {
                    readonly required: readonly ["x", "y"];
                };
            };
        };
        readonly '/session/:sessionId/touch/up': {
            readonly POST: {
                readonly command: "touchUp";
                readonly payloadParams: {
                    readonly required: readonly ["x", "y"];
                };
            };
        };
        readonly '/session/:sessionId/touch/move': {
            readonly POST: {
                readonly command: "touchMove";
                readonly payloadParams: {
                    readonly required: readonly ["x", "y"];
                };
            };
        };
        readonly '/session/:sessionId/touch/longclick': {
            readonly POST: {
                readonly command: "touchLongClick";
                readonly payloadParams: {
                    readonly required: readonly ["elements"];
                };
            };
        };
        readonly '/session/:sessionId/touch/flick': {
            readonly POST: {
                readonly command: "flick";
                readonly payloadParams: {
                    readonly optional: readonly ["element", "xspeed", "yspeed", "xoffset", "yoffset", "speed"];
                };
            };
        };
        readonly '/session/:sessionId/touch/perform': {
            readonly POST: {
                readonly command: "performTouch";
                readonly payloadParams: {
                    readonly wrap: "actions";
                    readonly required: readonly ["actions"];
                };
            };
        };
        readonly '/session/:sessionId/touch/multi/perform': {
            readonly POST: {
                readonly command: "performMultiAction";
                readonly payloadParams: {
                    readonly required: readonly ["actions"];
                    readonly optional: readonly ["elementId"];
                };
            };
        };
        readonly '/session/:sessionId/keys': {
            readonly POST: {
                readonly command: "keys";
                readonly payloadParams: {
                    readonly required: readonly ["value"];
                };
            };
        };
        readonly '/session/:sessionId/element/:elementId/location': {
            readonly GET: {
                readonly command: "getLocation";
            };
        };
        readonly '/session/:sessionId/element/:elementId/location_in_view': {
            readonly GET: {
                readonly command: "getLocationInView";
            };
        };
        readonly '/session/:sessionId/element/:elementId/size': {
            readonly GET: {
                readonly command: "getSize";
            };
        };
        readonly '/session/:sessionId/element/:elementId/equals/:otherId': {
            readonly GET: {
                readonly command: "equalsElement";
            };
        };
    };
    constructor(opts?: {}, shouldValidateCaps?: boolean);
    /** @type {boolean} */
    isProxyActive: boolean;
    /** @type {(toRun: {command?: string; script?: string}) => Promise<string>} */
    execPowerShell: (toRun: {
        command?: string;
        script?: string;
    }) => Promise<string>;
    /** @type {import('@appium/types').RouteMatcher[]} */
    jwpProxyAvoid: import("@appium/types").RouteMatcher[];
    desiredCapConstraints: {
        readonly platformName: {
            readonly presence: true;
            readonly isString: true;
            readonly inclusionCaseInsensitive: readonly ["Windows"];
        };
        readonly browserName: {
            readonly isString: true;
        };
        readonly platformVersion: {
            readonly isString: true;
        };
        readonly app: {
            readonly isString: true;
        };
        readonly appArguments: {
            readonly isString: true;
        };
        readonly appTopLevelWindow: {
            readonly isString: true;
        };
        readonly appWorkingDir: {
            readonly isString: true;
        };
        readonly createSessionTimeout: {
            readonly isNumber: true;
        };
        readonly 'ms:waitForAppLaunch': {
            readonly isNumber: true;
        };
        readonly 'ms:experimental-webdriver': {
            readonly isBoolean: true;
        };
        readonly systemPort: {
            readonly isNumber: true;
        };
        readonly prerun: {
            readonly isObject: true;
        };
        readonly postrun: {
            readonly isObject: true;
        };
    };
    resetState(): void;
    winAppDriver: WinAppDriver | null | undefined;
    _screenRecorder: any;
    createSession(...args: any[]): Promise<(string | import("@appium/types").DriverCaps<any>)[]>;
    startWinAppDriverSession(): Promise<void>;
    proxyReqRes: any;
    deleteSession(): Promise<void>;
    proxyActive(sessionId: any): boolean;
    canProxy(): boolean;
    getProxyAvoidList(sessionId: any): import("@appium/types").RouteMatcher[];
    proxyCommand(url: any, method: any, body?: null): Promise<unknown>;
}
import { desiredCapConstraints } from './desired-caps';
import { BaseDriver } from 'appium/driver';
import { WinAppDriver } from './winappdriver';
//# sourceMappingURL=driver.d.ts.map