"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.shellExec = shellExec;
exports.requireArgs = requireArgs;
const support_1 = require("appium/support");
const node_util_1 = require("node:util");
const node_child_process_1 = require("node:child_process");
const bluebird_1 = __importDefault(require("bluebird"));
const lodash_1 = __importDefault(require("lodash"));
const driver_1 = require("appium/driver");
const execAsync = (0, node_util_1.promisify)(node_child_process_1.exec);
/**
 * This API triggers UAC when necessary
 * unlike the 'spawn' call used by teen_process's exec.
 * See https://github.com/nodejs/node-v0.x-archive/issues/6797
 *
 * @param {string} cmd
 * @param {string[]} args
 * @param {import('node:child_process').ExecOptions & {timeoutMs?: number}} opts
 * @returns {Promise<{stdout: string; stderr: string;}>}
 * @throws {import('node:child_process').ExecException}
 */
async function shellExec(cmd, args = [], opts = {}) {
    const { timeoutMs = 60 * 1000 * 5 } = opts;
    const fullCmd = support_1.util.quote([cmd, ...args]);
    return await bluebird_1.default.resolve(execAsync(fullCmd, opts))
        .timeout(timeoutMs, `The command '${fullCmd}' timed out after ${timeoutMs}ms`);
}
/**
 * Assert the presence of particular keys in the given object
 *
 * @template {Object} T
 * @param {keyof T|(keyof T)[]} argNames one or more key names
 * @param {T} opts the object to check
 * @returns {T} the same given object
 */
function requireArgs(argNames, opts) {
    for (const argName of (lodash_1.default.isArray(argNames) ? argNames : [argNames])) {
        if (!lodash_1.default.has(opts, argName)) {
            throw new driver_1.errors.InvalidArgumentError(`'${String(argName)}' argument must be provided`);
        }
    }
    return opts;
}
//# sourceMappingURL=utils.js.map